@@ -4,10 +4,8 @@ require 5.005;  # 4-arg substr
 
 use strict;
 use vars qw($VERSION);
-use URI;
 
-$VERSION = "6.08_002";
-$VERSION = eval $VERSION;
+$VERSION = "6.00";
 
 my $CRLF = "\015\012";   # "\r\n" is not portable
 
@@ -38,7 +36,7 @@ sub http_configure {
     my($self, $cnf) = @_;
 
     die "Listen option not allowed" if $cnf->{Listen};
-    my $explicit_host = (exists $cnf->{Host});
+    my $explict_host = (exists $cnf->{Host});
     my $host = delete $cnf->{Host};
     my $peer = $cnf->{PeerAddr} || $cnf->{PeerHost};
     if (!$peer) {
@@ -46,30 +44,20 @@ sub http_configure {
 	$cnf->{PeerAddr} = $peer = $host;
     }
 
-    # CONNECTIONS
-    # PREFER: port number from PeerAddr, then PeerPort, then http_default_port
-    my $peer_uri = URI->new("http://$peer");
-    $cnf->{"PeerPort"} =  $peer_uri->_port || $cnf->{PeerPort} ||  $self->http_default_port;
-    $cnf->{"PeerAddr"} = $peer_uri->host;
+    if ($peer =~ s,:(\d+)$,,) {
+	$cnf->{PeerPort} = int($1);  # always override
+    }
+    if (!$cnf->{PeerPort}) {
+	$cnf->{PeerPort} = $self->http_default_port;
+    }
 
-    # HOST header:
-    # If specified but blank, ignore.
-    # If specified with a value, add the port number
-    # If not specified, set to PeerAddr and port number
-    # ALWAYS: If IPv6 address, use [brackets]  (thanks to the URI package)
-    # ALWAYS: omit port number if http_default_port
-    if (($host) || (! $explicit_host)) {
-        my $uri =  ($explicit_host) ? URI->new("http://$host") : $peer_uri->clone;
-        if (!$uri->_port) {
-            # Always use *our*  $self->http_default_port  instead of URI's  (Covers HTTP, HTTPS)
-            $uri->port( $cnf->{PeerPort} ||  $self->http_default_port);
-        }
-        my $host_port = $uri->host_port;               # Returns host:port or [ipv6]:port
-        my $remove = ":" . $self->http_default_port;   # we want to remove the default port number
-        if (substr($host_port,0-length($remove)) eq $remove) {
-            substr($host_port,0-length($remove)) = "";
-        }
-        $host = $host_port;
+    if (!$explict_host) {
+	$host = $peer;
+	$host =~ s/:.*//;
+    }
+    if ($host && $host !~ /:/) {
+	my $p = $cnf->{PeerPort};
+	$host .= ":$p" if $p != $self->http_default_port;
     }
 
     $cnf->{Proto} = 'tcp';
@@ -245,7 +233,6 @@ sub my_read {
 	    return length($_[0]);
 	}
 	else {
-	    die "read timeout" unless $self->can_read;
 	    return $self->sysread($_[0], $len);
 	}
     }
@@ -266,34 +253,25 @@ sub my_readline {
 		if $max_line_length && length($_) > $max_line_length;
 
 	    # need to read more data to find a line ending
-            my $new_bytes = 0;
-
           READ:
-            {   # wait until bytes start arriving
-                $self->can_read
-                     or die "read timeout";
-
-                # consume all incoming bytes
-                while(1) {
-                    my $bytes_read = $self->sysread($_, 1024, length);
-                    if(defined $bytes_read) {
-                        $new_bytes += $bytes_read;
-                        last if $bytes_read < 1024;
-                    }
-                    elsif($!{EINTR} || $!{EAGAIN} || $!{EWOULDBLOCK}) {
+            {
+                my $n = $self->sysread($_, 1024, length);
+                unless (defined $n) {
+                    redo READ if $!{EINTR};
+                    if ($!{EAGAIN}) {
+                        # Hmm, we must be reading from a non-blocking socket
+                        # XXX Should really wait until this socket is readable,...
+                        select(undef, undef, undef, 0.1);  # but this will do for now
                         redo READ;
                     }
-                    else {
-                        # if we have already accumulated some data let's at
-                        # least return that as a line
-                        length or die "$what read failed: $!";
-                        last;
-                    }
+                    # if we have already accumulated some data let's at least
+                    # return that as a line
+                    die "$what read failed: $!" unless length;
+                }
+                unless ($n) {
+                    return undef unless length;
+                    return substr($_, 0, length, "");
                 }
-
-                # no line-ending, no new bytes
-                return length($_) ? substr($_, 0, length($_), "") : undef
-                    if $new_bytes==0;
             }
 	}
 	die "$what line too long ($pos; limit is $max_line_length)"
@@ -306,38 +284,6 @@ sub my_readline {
 }
 
 
-sub can_read {
-    my $self = shift;
-    return 1 unless defined(fileno($self));
-    return 1 if $self->isa('IO::Socket::SSL') && $self->pending;
-
-    # With no timeout, wait forever.  An explicit timeout of 0 can be
-    # used to just check if the socket is readable without waiting.
-    my $timeout = @_ ? shift : (${*$self}{io_socket_timeout} || undef);
-
-    my $fbits = '';
-    vec($fbits, fileno($self), 1) = 1;
-  SELECT:
-    {
-        my $before;
-        $before = time if $timeout;
-        my $nfound = select($fbits, undef, undef, $timeout);
-        if ($nfound < 0) {
-            if ($!{EINTR} || $!{EAGAIN}) {
-                # don't really think EAGAIN can happen here
-                if ($timeout) {
-                    $timeout -= time - $before;
-                    $timeout = 0 if $timeout < 0;
-                }
-                redo SELECT;
-            }
-            die "select failed: $!";
-        }
-        return $nfound > 0;
-    }
-}
-
-
 sub _rbuf {
     my $self = shift;
     if (@_) {
@@ -469,7 +415,6 @@ sub read_entity_body {
 	    my @te = split(/\s*,\s*/, lc($te));
 	    die "Chunked must be last Transfer-Encoding '$te'"
 		unless pop(@te) eq "chunked";
-	    pop(@te) while @te && $te[-1] eq "chunked";  # ignore repeated chunked spec
 
 	    for (@te) {
 		if ($_ eq "deflate" && inflate_ok()) {
@@ -544,7 +489,6 @@ sub read_entity_body {
 		die "Bad chunk-size in HTTP response: $line";
 	    }
 	    $chunked = hex($1);
-	    ${*$self}{'http_chunked'} = $chunked;
 	    if ($chunked == 0) {
 		${*$self}{'http_trailers'} = [$self->_read_header_lines];
 		$$buf_ref = "";
@@ -592,7 +536,8 @@ sub read_entity_body {
 	my $n = $bytes;
 	$n = $size if $size && $size < $n;
 	$n = my_read($self, $$buf_ref, $n);
-	${*$self}{'http_bytes'} = defined $n ? $bytes - $n : $bytes;
+	return undef unless defined $n;
+	${*$self}{'http_bytes'} = $bytes - $n;
 	return $n;
     }
     else {
